@@ -1,7 +1,7 @@
 /* mpfr_mul_2ui -- multiply a floating-point number by a power of two
 
-Copyright 1999, 2001-2015 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 1999, 2001-2022 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -17,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include "mpfr-impl.h"
@@ -33,33 +33,19 @@ mpfr_mul_2ui (mpfr_ptr y, mpfr_srcptr x, unsigned long int n, mpfr_rnd_t rnd_mod
      ("y[%Pu]=%.*Rg inexact=%d",
       mpfr_get_prec (y), mpfr_log_prec, y, inexact));
 
-  inexact = MPFR_UNLIKELY(y != x) ? mpfr_set (y, x, rnd_mode) : 0;
+  inexact = (y != x) ? mpfr_set (y, x, rnd_mode) : 0;
 
-  if (MPFR_LIKELY( MPFR_IS_PURE_FP(y)) )
+  if (MPFR_LIKELY (MPFR_IS_PURE_FP (y)))
     {
-      /* n will have to be casted to long to make sure that the addition
-         and subtraction below (for overflow detection) are signed */
-      while (MPFR_UNLIKELY(n > LONG_MAX))
-        {
-          int inex2;
+      mpfr_exp_t exp = MPFR_GET_EXP (y);
+      mpfr_exp_t diffexp;
 
-          n -= LONG_MAX;
-          inex2 = mpfr_mul_2ui(y, y, LONG_MAX, rnd_mode);
-          if (inex2)
-            return inex2; /* overflow */
-        }
-
-      /* MPFR_EMIN_MIN + (long) n is signed and doesn't lead to an overflow;
-         the first test useful so that the real test can't lead to an
-         overflow. */
-      {
-        mpfr_exp_t exp = MPFR_GET_EXP (y);
-        if (MPFR_UNLIKELY( __gmpfr_emax < MPFR_EMIN_MIN + (long) n ||
-                           exp > __gmpfr_emax - (long) n))
-          return mpfr_overflow (y, rnd_mode, MPFR_SIGN(y));
-
-        MPFR_SET_EXP (y, exp + (long) n);
-      }
+      diffexp = __gmpfr_emax - exp;  /* diff of two valid exponents */
+      if (MPFR_UNLIKELY (n > diffexp))  /* exp + n > emax */
+        return mpfr_overflow (y, rnd_mode, MPFR_SIGN (y));
+      /* Now, 0 <= n <= diffexp, thus n fits in a mpfr_exp_t,
+         and exp + n <= emax (no integer overflow). */
+      MPFR_SET_EXP (y, exp + (mpfr_exp_t) n);
     }
 
   return inexact;
