@@ -1,6 +1,6 @@
 /* Test file for multiple mpfr.h inclusion and va_list related functions
 
-Copyright 2011-2017 Free Software Foundation, Inc.
+Copyright 2011-2022 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -17,10 +17,14 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#if HAVE_STDARG
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#if defined(HAVE_STDARG) && !defined(MPFR_USE_MINI_GMP)
 
 #if _MPFR_EXP_FORMAT == 4
 /* If mpfr_exp_t is defined as intmax_t, intmax_t must be defined before
@@ -28,11 +32,6 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 # include <stdint.h>
 #endif
 
-#define MPFR_DONT_USE_FILE  /* don't use FILE in mpfr-impl.h */
-#ifdef MPFR_USE_MINI_GMP
-#include "mpfr-test.h"
-#endif
-
 /* One of the goals of this test is to detect potential issues with the
  * following case in user code:
  *
@@ -47,8 +46,8 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
  * the second inclusion is not a no-op due to some #include guard. This
  * was fixed in r7320.
  *
- * With mini-gmp, mpfr-impl.h is included first, but this should not
- * affect this test thanks to MPFR_DONT_USE_FILE.
+ * Note: one needs to make sure that optimizations do not drop the call
+ * to mpfr_vfprintf.
  */
 #include <mpfr.h>
 
@@ -65,23 +64,18 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 static void
 test (FILE *fout, const char *fmt, ...)
 {
-  int (*fct) (FILE*, const char*, va_list);
+  va_list ap;
 
-  fct = mpfr_vfprintf;
-  if (0)
-    {
-      va_list ap;
-      va_start (ap, fmt);
-      fct (fout, fmt, ap);
-      va_end (ap);
-    }
+  va_start (ap, fmt);
+  mpfr_vfprintf (fout, fmt, ap);
+  va_end (ap);
 }
 
 int
 main (void)
 {
   tests_start_mpfr ();
-  test (stdout, "%d\n", 0);
+  test (stdout, "");
   tests_end_mpfr ();
   return 0;
 }
